
/***************************************************
* By CCSC compiler
*  File Name: 2000627_MK322mod_k303_talking_alart_1823.c
*  Rev 1.3
*  Description: talking distance meter
*  RC1 out /reset for ATP
*  RC2 out RXD for ATP
*  RC3 in PLAY
*  RC4 in SETTING
*  RC5 out SENSOR trigger
*  RA3 in SENSOR echo
*  RA4 in AD threshold (0 to 5V)
*  COPYRIGHT 2020 MYCOMKITS.COM, owned by CNET LIMITED
̃vO̒쌠́Aҁu}CRLbghbgR^c@LЃNlbgvɋA܂B
X͒쌠Ă܂񂪁AvOgwK̒ŃvORɕύXĂgB
The copyright of this program belongs to the producer "MYCOMKITS.COMhmanagement company CNET LIMITED". 
We do not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16f1823.h>
#DEVICE ADC=8
#fuses	INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses	NOCPD, NOCLKOUT, NOWRT, NOIESO, NOFCMEN, PLL_SW, NOSTVREN
#use	delay(CLOCK = 4000000)

#use	rs232(BAUD=9600, XMIT=PIN_C2)
//
#use	fast_io(a)
#use	fast_io(c)
//
#define Amode 0xFB			//port A all input
#define Cmode 0xD8			//port C all input except 1, 2, 5
//
#byte OSCCON = 0x099
#byte WPUC = 0x20E
#byte OPTION_REG = 0x095
#byte PCON = 0x096
#bit T2IF = 0x011.1 //PIR1

// HC-SR04 distance sensor, two signal
#define trig PIN_C5
#define echo PIN_A3

#define LED0 PIN_A2 //active low
#define LED1 PIN_C0 //active low

// variables
float numberf = 0;                 // 7 seg data
long repeat_time = 500;
long value_setting = 50;	// max 256
float coeff = 0.00017;// (340/2)/1000000
float value_s[4];
int error = 0;
int count=0;

//
#separate
void hcsr04_read();
#separate
void initializing();
#separate
void tellvalue(); 
#separate
void get_setting();

// timer 1 sub function
// wait time count down the timer value at every 100msec.
#int_timer0
void isr0(void)	
{
	set_timer0(0x3D);             // setting timer 0. almost 50msec
	if(repeat_time > 0) repeat_time = repeat_time -1;
	else repeat_time = 0;
}
// main function
void main()
{
	initializing();
    output_low(LED0);
    output_high(LED1);
	// main loop
	while(1)
	{
		//
		if(input(PIN_C4) == 0)	// set threshold value
		{
			get_setting();	// get value_setting from potentiometer
			if(value_setting >235)
			{
				disable_interrupts(GLOBAL);
				//
				putc(0x24);	// send brake command $
				printf("ripi'-to/shimase'nn.");
				putc(0x0D);
				//
				enable_interrupts(GLOBAL);
				delay_ms(3000);
			}
			else
			{	
				numberf = value_setting/2;
				//
				disable_interrupts(GLOBAL);
				putc(0x24);	// send brake command $
				putc(0x3C);
				printf("NUMK VAL=");
				printf("%2.1f",numberf);
				putc(0x3E);
				printf("biyo'-");	//biyou, byo'-
				putc(0x0D);
				//
				enable_interrupts(GLOBAL);
				delay_ms(3000);
			}
		}
		else
		{

            hcsr04_read(); 	// get numberf, distance from HCSR04
            delay_ms(60);
                //
			if((repeat_time == 0) && (value_setting <=235))	// repeat time will be decremented by int1
			{

				// play
				if(error == 1) 
				{
					// tell error
					
				}
				else
				{	
					putc(0x24);	// send brake command $
                    delay_ms(200);
					tellvalue();
					repeat_time = value_setting;	// 5sec = 50x50msec repeat cycle
					repeat_time = repeat_time * 10; //value_setting(actual time x2)x10
					delay_ms(60);
				}
			}	
			// play it, if pressing PLAY switch
			if(input(PIN_C3) == 0)
			{
				delay_ms(50);
				if(input(PIN_C3) == 0)
				{
					// play
					putc(0x24);	// send brake command $
                    delay_ms(200);
					tellvalue();
					delay_ms(60);
					while(input(PIN_C3) == 0) {}
				}	
			}
		}		
	}	
}
//
#separate
void initializing() 
{
	//
	OSCCON = 0xEA;	// set 4MHz
	//
	set_tris_a(Amode);	//
	set_tris_c(Cmode);	//
	//
	//OPTION_REG = OPTION_REG & 0x7F;	// set 0 to WPUEN
	//WPUC = 0x18;	// pull up RC3, RC4
	PCON = 0x05;	//
    port_a_pullups(0x18);   
    output_high(PIN_C0);    //LED1
    output_high(PIN_A2);    //LED0
	//
	setup_comparator(NC_NC_NC_NC);
	SETUP_DAC(DAC_OFF);
	// A/D converter initialize 
	setup_adc_ports(sAN3 | VSS_VDD);	// RA4, gnd to 5.0V
	setup_adc(ADC_CLOCK_DIV_8);	// 2usec	
	// initialize timer 1 for counting distance
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);	//Fosc/4, 1:1, Enables Timer1, resolution 1us(1MHz)
	set_timer1(0x0);  
	// initialize timer 1 for repeat play
	setup_timer_0(T0_INTERNAL|T0_DIV_256);	// 256us cycle
	set_timer0(0x3D);              // setting almost 50msec
    
    setup_timer_2(T2_DIV_BY_64, 0xFF, 16);	//Fosc/4, 1:64, resolution 1ms(1kHz)
	set_timer2(0x0); 
	// reset ATP3011
	output_high(PIN_C1);	// ATP RESET PORT
	output_low(PIN_C1);
	delay_ms(500);
	output_high(PIN_C1);
	delay_ms(100);
	//
	printf("?");	//set baud rate
	delay_ms(100);

	//printf("to-kinngume'ja-/kido-shima'_shita.");
    printf("sekkin/keikokuso'uti-/kido-shima'_shita.");
	putc(0x0D);
    delay_ms(5000);
	//
    int i;
    for(i=0;i<4;i++)
    {
        hcsr04_read();
        delay_ms(100);
    }
    
	//delay_ms(4000);
	//
	get_setting();
	//
	
	enable_interrupts(INT_TIMER0);   // enable timer 0
	enable_interrupts(GLOBAL);
	//
}	
//
#separate
void hcsr04_read()
{
	/* moving average
	int i;
	for (i=3; i>0; i--)
	{
		value_s[i] = value_s[i-1];
	}
	// trigger
      */
	output_high(PIN_C5);
	delay_us(10);
	output_low(PIN_C5);
	//
	while(input(PIN_A3)==0) {}
	disable_interrupts(GLOBAL);
	set_timer1(0);
	while(input(PIN_A3)==1)
	{
		if(get_timer1()>40000) 
		{
			error=1;
			break;
		}	
		else error=0;
	}
	//
	numberf = coeff * get_timer1();
	/*
	value_s[0] = numberf;
	//numberf = 0;
	for (i=1; i<4; i++)
	{
		numberf = numberf + value_s[i];
	}
	numberf = numberf / 4;
	*/ 
	enable_interrupts(GLOBAL);
	//
} 
//
#separate
void tellvalue() 
{
	disable_interrupts(GLOBAL);
	if(numberf >1)
	{
		if(numberf <2)
        {
        putc(0x3C);
		printf("NUMK VAL=");
		printf("%1.2f",numberf);
		putc(0x3E);
        printf("me'-ta-.nime-ta-i'jyo-/hana'retekudasai.");
		//printf("me'-ta-..2me-ta-ijyou/hana'retekudasa'i..2me-ta-ijyou/hana'retekudasai.");
		putc(0x0D);
        output_high(LED0);
        output_low(LED1);
        delay_ms(8000);
        }
        else
        {
            output_low(LED0);
            output_high(LED1);
        }
	}
	else
	{
		numberf = numberf * 100;
		putc(0x3C);
		printf("NUMK VAL=");
		printf("%2.2f",numberf);
		putc(0x3E);
        printf("sennchime'-ta-..tika'sugimasu.hana'retekudasai.");
		putc(0x0D);
        output_low(LED0);
        output_low(LED1);
        delay_ms(8000);
	}	
	enable_interrupts(GLOBAL);		
}	

// get repeat time value from potentiometer
#separate
void get_setting()
{
	//
	disable_interrupts(GLOBAL);
	//
	set_adc_channel(3);	//must wait 65u, AN3, RA4
	delay_us(100);
	value_setting = read_adc();	// 0 to 255, 0V to 5V
	if(value_setting <= 10) value_setting = 2;	// 1 second
    //value_setting = 6;  //3second fixed
	repeat_time = value_setting/2;	// 0 to 127 = 0 to 127 sec
	//
	enable_interrupts(GLOBAL);
}

